@@ -1,22 +1,10 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2009-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2009-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-Kajongg is free software you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation either version 2 of the License, or
-(at your option) any later version.
+SPDX-License-Identifier: GPL-2.0
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 import traceback
@@ -26,9 +14,10 @@ import weakref
 from twisted.spread import pb
 from twisted.internet.defer import Deferred
 
-from log import m18nE, logInfo, logDebug, logException
+from log import logInfo, logDebug, logException, id4
+from mi18n import i18nE
 from message import Message
-from common import Debug, StrMixin, unicode, nativeString, nativeStringArgs
+from common import Debug, StrMixin
 from move import Move
 
 
@@ -50,62 +39,58 @@ class Request(StrMixin):
     @property
     def block(self):
         """hide weakref"""
-        if self._block:
-            return self._block()
+        return self._block() if self._block else None
 
     @property
     def user(self):
         """hide weakref"""
-        if self._user:
-            return self._user()
+        return self._user() if self._user else None
 
     @property
     def about(self):
         """hide weakref"""
-        if self._about:
-            return self._about()
+        return self._about() if self._about else None
 
     @property
     def player(self):
         """hide weakref"""
-        if self._player:
-            return self._player()
+        return self._player() if self._player else None
 
     def gotAnswer(self, rawAnswer):
         """convert the wired answer into something more useful"""
         if isinstance(rawAnswer, tuple):
-            answer = nativeString(rawAnswer[0])
+            answer = rawAnswer[0]
             if isinstance(rawAnswer[1], tuple):
-                self.args = nativeStringArgs(rawAnswer[1])
+                self.args = rawAnswer[1]
             else:
-                self.args = nativeStringArgs([rawAnswer[1]])
+                self.args = [rawAnswer[1]]
         else:
-            answer = nativeString(rawAnswer)
+            answer = rawAnswer
             self.args = None
         if answer in Message.defined:
             self.answer = Message.defined[answer]
         else:
             if Debug.deferredBlock:
-                logDebug(u'Request %s ignores %s' % (self, rawAnswer))
+                logDebug('Request %s ignores %s' % (self, rawAnswer))
 
     def age(self):
         """my age in full seconds"""
         return int((datetime.datetime.now() - self.startTime).total_seconds())
 
-    def __unicode__(self):
+    def __str__(self):
         cmd = self.deferred.command
         if self.answer:
-            answer = unicode(self.answer)
+            answer = str(self.answer) # TODO: needed?
         else:
-            answer = u'OPEN'
-        result = u''
+            answer = 'OPEN'
+        result = ''
         if Debug.deferredBlock:
-            result += u'[{id:>4}] '.format(id=id(self) % 10000)
-        result += u'{cmd}->{cls}({receiver:<10}): {answer}'.format(
+            result += '[{id4:>4}] '.format(id4=id4(self))
+        result += '{cmd}->{cls}({receiver:<10}): {answer}'.format(
             cls=self.user.__class__.__name__, cmd=cmd, receiver=self.user.name,
             answer=answer)
         if self.age():
-            result += u' after {} sec'.format(self.age())
+            result += ' after {} sec'.format(self.age())
         return result
 
     def prettyAnswer(self):
@@ -120,14 +105,14 @@ class Request(StrMixin):
 
     def pretty(self):
         """for debug output"""
-        result = u''
+        result = ''
         if Debug.deferredBlock:
-            result += u'[{id:>4}] '.format(id=id(self) % 10000)
-        result += u'{cmd:<12}<-{cls:>6}({receiver:<10}): ANS={answer}'.format(
+            result += '[{id4:>4}] '.format(id4=id4(self))
+        result += '{cmd:<12}<-{cls:>6}({receiver:<10}): ANS={answer}'.format(
             cls=self.user.__class__.__name__,
             answer=self.prettyAnswer(), cmd=self.deferred.command, receiver=self.user.name)
         if self.age() > 0:
-            result += u' after {} sec'.format(self.age())
+            result += ' after {} sec'.format(self.age())
         return result
 
 
@@ -161,28 +146,28 @@ class DeferredBlock(StrMixin):
                     for block in DeferredBlock.blocks:
                         logInfo(str(block))
 
-    def debugPrefix(self, marker=u''):
+    def debugPrefix(self, dbgMarker=''):
         """prefix for debug message"""
-        return u'T{table} B[{id:>4}] {caller:<15} {marker:<3}(out={out})'.format(
-            table=self.table.tableid, id=id(self) % 10000, caller=self.calledBy[:15],
-            marker=marker, out=self.outstanding)
+        return 'T{table} B[{id4:>4}] {caller:<15} {dbgMarker:<3}(out={out})'.format(
+            table=self.table.tableid, id4=id4(self), caller=self.calledBy[:15],
+            dbgMarker=dbgMarker, out=self.outstanding)
 
-    def debug(self, marker, msg):
+    def debug(self, dbgMarker, msg):
         """standard debug format"""
-        logDebug(u' '.join([self.debugPrefix(marker), msg]))
+        logDebug(' '.join([self.debugPrefix(dbgMarker), msg]))
 
-    def __unicode__(self):
-        return u'%s requests=%s outstanding=%d %s callback=%s' % (
+    def __str__(self):
+        return '%s requests=%s outstanding=%d %s callback=%s' % (
             self.debugPrefix(),
-            u'[' + u','.join(unicode(x) for x in self.requests) + u']',
+            '[' + ','.join(str(x) for x in self.requests) + ']',
             self.outstanding,
-            u'is completed' if self.completed else u'not completed',
+            'is completed' if self.completed else 'not completed',
             self.prettyCallback())
 
-    def outstandingUnicode(self):
-        """like __unicode__ but only with outstanding answers"""
-        return u'%s callback=%s:%s' % (self.calledBy, self.prettyCallback(),
-                                       u'[' + u','.join(str(x) for x in self.requests if not x.answer) + u']')
+    def outstandingStr(self):
+        """like __str__ but only with outstanding answers"""
+        return '%s callback=%s:%s' % (self.calledBy, self.prettyCallback(),
+                                      '[' + ','.join(str(x) for x in self.requests if not x.answer) + ']')
 
     @staticmethod
     def garbageCollection():
@@ -191,12 +176,12 @@ class DeferredBlock(StrMixin):
         never overlaps."""
         for block in DeferredBlock.blocks[:]:
             if block.callbackMethod is None:
-                block.logBug(u'DBlock %s has no callback' % str(block))
+                block.logBug('DBlock %s has no callback' % str(block))
             if block.completed:
                 DeferredBlock.blocks.remove(block)
         if len(DeferredBlock.blocks) > 100:
             logDebug(
-                u'We have %d DeferredBlocks, they must be leaking' %
+                'We have %d DeferredBlocks, they must be leaking' %
                 len(DeferredBlock.blocks))
 
     def __addRequest(self, deferred, user, about):
@@ -213,9 +198,9 @@ class DeferredBlock(StrMixin):
                 request)
         if Debug.deferredBlock:
             notifying = ' notifying' if deferred.notifying else ''
-            rqString = u'[{id:>4}] {cmd}{notifying} {about}->{cls:>6}({receiver:<10})'.format(
+            rqString = '[{id4:>4}] {cmd}{notifying} {about}->{cls:>6}({receiver:<10})'.format(
                 cls=user.__class__.__name__,
-                id=id(request) % 10000, cmd=deferred.command, receiver=user.name,
+                id4=id4(request), cmd=deferred.command, receiver=user.name,
                 about=about.name if about else '', notifying=notifying)
             self.debug('+:%d' % len(self.requests), rqString)
 
@@ -225,7 +210,7 @@ class DeferredBlock(StrMixin):
         if not request.answer:
             self.outstanding -= 1
         if Debug.deferredBlock:
-            self.debug(u'-:%d' % self.outstanding, unicode(request))
+            self.debug('-:%d' % self.outstanding, str(request)) # TODO: auch ohne?
         self.callbackIfDone()
 
     def callback(self, method, *args):
@@ -247,22 +232,21 @@ class DeferredBlock(StrMixin):
                 if Debug.deferredBlock:
                     self.debug('IGN', request.pretty())
                 return
-            else:
-                request.gotAnswer(result)
-                if hasattr(request.user, 'pinged'):
-                    # a Client (for robots) does not have it
-                    request.user.pinged()
-                if Debug.deferredBlock:
-                    self.debug('ANS', request.pretty())
-                if hasattr(request.answer, 'notifyAction'):
-                    block = DeferredBlock(self.table, temp=True)
-                    receivers = request.answer.receivers(request)
-                    if receivers:
-                        block.tell(
-                            request.player,
-                            receivers,
-                            request.answer,
-                            notifying=True)
+            request.gotAnswer(result)
+            if hasattr(request.user, 'pinged'):
+                # a Client (for robots) does not have it
+                request.user.pinged()
+            if Debug.deferredBlock:
+                self.debug('ANS', request.pretty())
+            if hasattr(request.answer, 'notifyAction'):
+                block = DeferredBlock(self.table, temp=True)
+                receivers = request.answer.receivers(request)
+                if receivers:
+                    block.tell(
+                        request.player,
+                        receivers,
+                        request.answer,
+                        notifying=True)
             self.outstanding -= 1
             assert self.outstanding >= 0, '__gotAnswer: outstanding %d' % self.outstanding
             self.callbackIfDone()
@@ -275,17 +259,14 @@ class DeferredBlock(StrMixin):
         if request in self.requests:
             self.removeRequest(request)
         if result.type in [pb.PBConnectionLost]:
-            msg = m18nE('The game server lost connection to player %1')
+            msg = i18nE('The game server lost connection to player %1')
             self.table.abort(msg, request.user.name)
         else:
-            msg = m18nE('Error for player %1: %2\n%3')
+            msg = i18nE('Error for player %1: %2\n%3')
             if hasattr(result, 'traceback'):
                 traceBack = result.traceback
             else:
-                try:
-                    traceBack = result.getBriefTraceback()
-                except BaseException as exc:
-                    traceBack = u'twisted cannot give us a traceback:{}'.format(exc)
+                traceBack = result.getBriefTraceback()
             self.table.abort(
                 msg,
                 request.user.name,
@@ -295,7 +276,7 @@ class DeferredBlock(StrMixin):
     def logBug(self, msg):
         """log msg and raise exception"""
         for request in self.requests:
-            logDebug(unicode(request))
+            logDebug(str(request)) # TODO:
         logException(msg)
 
     def callbackIfDone(self):
@@ -307,52 +288,52 @@ class DeferredBlock(StrMixin):
             self.completed = True
             if any(not x.answer for x in self.requests):
                 self.logBug(
-                    u'Block %s: Some requests are unanswered' %
+                    'Block %s: Some requests are unanswered' %
                     str(self))
             if Debug.deferredBlock:
                 commandText = []
-                for command in sorted(set(x.deferred.command for x in self.requests)):
-                    text = u'%s:' % command
+                for command in sorted({x.deferred.command for x in self.requests}):
+                    text = '%s:' % command
                     answerList = []
-                    for answer in sorted(set(x.prettyAnswer() for x in self.requests if x.deferred.command == command)):
-                        answerList.append((answer, list(
+                    for answer in sorted({x.prettyAnswer() for x in self.requests if x.deferred.command == command}):
+                        answerList.append((answer, [
                             x for x in self.requests
-                            if x.deferred.command == command and answer == x.prettyAnswer())))
+                            if x.deferred.command == command and answer == x.prettyAnswer()]))
                     answerList = sorted(answerList, key=lambda x: len(x[1]))
                     answerTexts = []
                     if len(answerList) == 1:
                         answerTexts.append(
-                            u'{answer} from all'.format(answer=answerList[-1][0]))
+                            '{answer} from all'.format(answer=answerList[-1][0]))
                     else:
                         for answer, requests in answerList[:-1]:
                             answerTexts.append(
-                                u'{answer} from {players}'.format(answer=answer,
-                                                                  players=u','.join(x.user.name for x in requests)))
+                                '{answer} from {players}'.format(answer=answer,
+                                                                 players=','.join(x.user.name for x in requests)))
                         answerTexts.append(
-                            u'{answer} from others'.format(answer=answerList[-1][0]))
-                    text += u', '.join(answerTexts)
+                            '{answer} from others'.format(answer=answerList[-1][0]))
+                    text += ', '.join(answerTexts)
                     commandText.append(text)
                 methodName = self.prettyCallback()
                 if methodName:
-                    methodName = u' next:%s' % methodName
+                    methodName = ' next:%s' % methodName
                 self.debug(
-                    u'END',
-                    u'{answers} {method}'.format(method=methodName,
-                                                 answers=u' / '.join(commandText)))
+                    'END',
+                    '{answers} {method}'.format(method=methodName,
+                                                answers=' / '.join(commandText)))
             if self.callbackMethod is not False:
                 self.callbackMethod(self.requests, *self.__callbackArgs)
 
     def prettyCallback(self):
         """pretty string for callbackMethod"""
         if self.callbackMethod is False:
-            result = u''
+            result = ''
         elif self.callbackMethod is None:
-            result = u'None'
+            result = 'None'
         else:
             result = self.callbackMethod.__name__
             if self.__callbackArgs:
-                result += u'({})'.format(
-                    u','.join([str(x) for x in self.__callbackArgs] if self.__callbackArgs else u''))
+                result += '({})'.format(
+                    ','.join([str(x) for x in self.__callbackArgs] if self.__callbackArgs else ''))
         return result
 
     def playerForUser(self, user):
@@ -363,6 +344,7 @@ class DeferredBlock(StrMixin):
             for player in self.table.game.players:
                 if user.name == player.name:
                     return player
+        return None
 
     @staticmethod
     def __enrichMessage(game, about, command, kwargs):
@@ -390,10 +372,11 @@ class DeferredBlock(StrMixin):
         """send info about player 'about' to users 'receivers'"""
         def encodeKwargs():
             """those values are classes like Meld, Tile etc.
-               Convert to str(python2) or bytes(python3"""
-            for keyword in ('tile', 'tiles', 'meld', 'melds'):
-                if keyword in kwargs:
-                    kwargs[keyword] = str(kwargs[keyword]).encode()
+               Convert to bytes"""
+            for keyword in kwargs:
+                if any(keyword.lower().endswith(x) for x in ('tile', 'tiles', 'meld', 'melds')):
+                    if kwargs[keyword] is not None:
+                        kwargs[keyword] = str(kwargs[keyword])
         encodeKwargs()
         if about.__class__.__name__ == 'User':
             about = self.playerForUser(about)
@@ -409,31 +392,36 @@ class DeferredBlock(StrMixin):
             self.table.game.appendMove(about, command, kwargs)
         localDeferreds = []
         for rec in self.__convertReceivers(receivers):
+
             isClient = rec.__class__.__name__.endswith('Client')
-            if Debug.traffic and not isClient:
-                message = u'-> {receiver:<15} about {about} {command}{kwargs}'.format(
-                    receiver=rec.name[:15], about=about, command=command,
-                    kwargs=Move.prettyKwargs(kwargs))
-                logDebug(message)
             if isClient:
                 defer = Deferred()
                 defer.addCallback(rec.remote_move, command, **kwargs)
+                defer.command = command.name
+                defer.notifying = 'notifying' in kwargs
+                self.__addRequest(defer, rec, about)
+                localDeferreds.append(defer)
             else:
+                if Debug.traffic:
+                    message = '-> {receiver:<15} about {about} {command}{kwargs}'.format(
+                        receiver=rec.name[:15], about=about, command=command,
+                        kwargs=Move.prettyKwargs(kwargs))
+                    logDebug(message)
                 defer = self.table.server.callRemote(
                     rec,
                     'move',
                     aboutName,
                     command.name,
                     **kwargs)
-            if defer:
-                defer.command = command.name
-                defer.notifying = 'notifying' in kwargs
-                self.__addRequest(defer, rec, about)
-            else:
-                msg = m18nE('The game server lost connection to player %1')
-                self.table.abort(msg, rec.name)
-            if isClient:
-                localDeferreds.append(defer)
+                if defer:
+                    defer.command = command.name
+                    defer.notifying = 'notifying' in kwargs
+                    self.__addRequest(defer, rec, about)
+                else:
+                    msg = i18nE('The game server lost connection to player %1')
+                    self.table.abort(msg, rec.name)
+
+
         for defer in localDeferreds:
             defer.callback(aboutName)  # callback needs an argument !
 
@@ -446,7 +434,7 @@ class DeferredBlock(StrMixin):
         self.tell(
             player,
             list(
-                [x for x in self.table.game.players if x.name != player.name]),
+                x for x in self.table.game.players if x.name != player.name),
             command,
             **kwargs)
 
