@@ -3,19 +3,8 @@
 """
 Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-Kajongg is free software you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation either version 2 of the License, or
-(at your option) any later version.
+SPDX-License-Identifier: GPL-2.0
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 import datetime
@@ -44,7 +33,7 @@ class GamesModel(QAbstractTableModel):
         QAbstractTableModel.__init__(self)
         self._resultRows = []
 
-    def columnCount(self, dummyParent=None):   # pylint: disable=no-self-use
+    def columnCount(self, unusedParent=None):   # pylint: disable=no-self-use
         """including the hidden col 0"""
         return 3
 
@@ -69,6 +58,7 @@ class GamesModel(QAbstractTableModel):
                 or column < 0
                 or row >= self.rowCount(parent)
                 or column >= self.columnCount(parent)):
+	    # similar in tree.py
             return QModelIndex()
         return self.createIndex(row, column, 0)
 
@@ -77,7 +67,7 @@ class GamesModel(QAbstractTableModel):
         if role is None:
             role = Qt.DisplayRole
         if not (index.isValid() and role == Qt.DisplayRole):
-            return
+            return None
         if role == Qt.DisplayRole:
             unformatted = str(
                 self._resultRows[index.row()][index.column()]) # TODO: brauche ich str?
@@ -86,11 +76,11 @@ class GamesModel(QAbstractTableModel):
                 # this translation is needed for robot players
                 names = [i18n(name) for name in unformatted.split('///')]
                 return ', '.join(names)
-            elif index.column() == 1:
+            if index.column() == 1:
                 dateVal = datetime.datetime.strptime(
                     unformatted, '%Y-%m-%dT%H:%M:%S')
                 return dateVal.strftime('%c')
-            elif index.column() == 0:
+            if index.column() == 0:
                 return int(unformatted)
         return QAbstractTableModel.data(self, index, role)
 
@@ -98,7 +88,8 @@ class GamesModel(QAbstractTableModel):
         """for the two visible columns"""
         # pylint: disable=no-self-use
         if orientation == Qt.Horizontal and role == Qt.DisplayRole:
-            return (i18n("Started"), i18n("Players"))[section - 1]
+            return i18n('Players') if section == 2 else i18n('Started')
+        return None
 
 
 class Games(QDialog):
@@ -106,7 +97,7 @@ class Games(QDialog):
     """a dialog for selecting a game"""
 
     def __init__(self, parent=None):
-        super(Games, self).__init__(parent)
+        super().__init__(parent)
         self.selectedGame = None
         self.onlyPending = True
         decorateWindow(self, i18nc("@title:window", "Games"))
@@ -156,7 +147,7 @@ class Games(QDialog):
         self.view.doubleClicked.connect(self.loadGame)
         chkPending.stateChanged.connect(self.pendingOrNot)
 
-    def showEvent(self, dummyEvent):
+    def showEvent(self, unusedEvent):
         """only now get the data set. Not doing this in__init__ would eventually
         make it easier to subclass from some generic TableEditor class"""
         self.setQuery()
@@ -198,7 +189,7 @@ class Games(QDialog):
         self.view.hideColumn(0)
 
     def __idxForGame(self, game):
-        """returns the model index for game"""
+        """return the model index for game"""
         for row in range(self.model.rowCount()):
             idx = self.model.index(row, 0)
             if self.model.data(idx, 0) == game:
@@ -206,12 +197,9 @@ class Games(QDialog):
         return self.model.index(0, 0)
 
     def __getSelectedGame(self):
-        """returns the game id of the selected game"""
+        """return the game id of the selected game"""
         rows = self.selection.selectedRows()
-        if rows:
-            return self.model.data(rows[0], 0)
-        else:
-            return 0
+        return self.model.data(rows[0], 0) if rows else 0
 
     def pendingOrNot(self, chosen):
         """do we want to see all games or only pending games?"""
@@ -238,8 +226,8 @@ class Games(QDialog):
                     Query("DELETE FROM game WHERE id = ?", (game, ))
                 self.setQuery()  # just reload entire table
         allGames = self.view.selectionModel().selectedRows(0)
-        deleteGames = list(x.data() for x in allGames)
-        if len(deleteGames) == 0:
+        deleteGames = [x.data() for x in allGames]
+        if not deleteGames:
             # should never happen
             logException('delete: 0 rows selected')
         WarningYesNo(
