@@ -2,21 +2,7 @@
 
 """Copyright (C) 2009-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-Kajongg is free software you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation either version 2 of the License, or
-(at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
-
-
+SPDX-License-Identifier: GPL-2.0
 
 Read the user manual for a description of the interface to this scoring engine
 """
@@ -60,7 +46,7 @@ class Score(StrMixin):
         self.points = self.doubles = self.limits = 0
 
     def change(self, unitName, value):
-        """sets value for unitName. If changed, return True"""
+        """set value for unitName. If changed, return True"""
         oldValue = self.__getattribute__(unitName)
         newValue = type(oldValue)(value)
         if newValue == oldValue:
@@ -217,7 +203,7 @@ class RuleList(list):
                 rule.key(), self[rule.key()].definition, rule.name, rule.definition))
         self[rule.key()] = rule
 
-    def createRule(self, name: str, definition: str='', **kwargs):
+    def createRule(self, name: str, definition: str = '', **kwargs):
         """shortcut for simpler definition of predefined rulesets"""
         defParts = definition.split('||')
         rule = None
@@ -299,11 +285,6 @@ class Ruleset:
     hits = 0
     misses = 0
 
-    @staticmethod
-    def clearCache():
-        """clears the cache with Rulesets"""
-        Ruleset.cache.clear()
-
     @staticmethod
     def cached(name):
         """If a Ruleset instance is never changed, we can use a cache"""
@@ -403,7 +384,7 @@ into a situation where you have to pay a penalty"""))
 
     @staticmethod
     def hashIsKnown(value):
-        """returns False or True"""
+        """return False or True"""
         result = any(x.hash == value for x in PredefinedRuleset.rulesets())
         if not result:
             query = Query("select id from ruleset where hash=?", (value,))
@@ -425,14 +406,14 @@ into a situation where you have to pay a penalty"""))
             return
         else:
             query = Query("select id,hash,name,description from ruleset where hash=?", (self.name,))
-        if len(query.records):
+        if query.records:
             (self.rulesetId, self.__hash, self.name,
              self.description) = query.records[0]
         else:
             raise Exception('ruleset %s not found' % self.name)
 
     def __setParametersFrom(self, fromRuleset):
-        """sets attributes for parameters defined in fromRuleset.
+        """set attributes for parameters defined in fromRuleset.
         Does NOT overwrite already set parameters: Silently ignore them"""
         for par in fromRuleset.parameterRules:
             if isinstance(par, ParameterRule):
@@ -447,8 +428,8 @@ into a situation where you have to pay a penalty"""))
         self.loadRules()
         self.__setParametersFrom(self)
         for ruleList in self.ruleLists:
-            assert len(ruleList) == len(set(x.key()
-                                            for x in ruleList)), '%s has non-unique key' % ruleList.name
+            assert len(ruleList) == len({x.key()
+                                         for x in ruleList}), '%s has non-unique key' % ruleList.name
             for rule in ruleList:
                 if hasattr(rule, 'score'):
                     rule.score.ruleset = self
@@ -460,10 +441,8 @@ into a situation where you have to pay a penalty"""))
             self.__setParametersFrom(
                 sorted(PredefinedRuleset.rulesets(),
                        key=lambda x: len(self.diff(x)))[0])
-        self.doublingMeldRules = list(
-            x for x in self.meldRules if x.score.doubles)
-        self.doublingHandRules = list(
-            x for x in self.handRules if x.score.doubles)
+        self.doublingMeldRules = [x for x in self.meldRules if x.score.doubles]
+        self.doublingHandRules = [x for x in self.handRules if x.score.doubles]
         for mjRule in self.mjRules:
             if mjRule.__class__.__name__ == 'StandardMahJonggRule':
                 self.standardMJRule = mjRule
@@ -472,13 +451,13 @@ into a situation where you have to pay a penalty"""))
         return self
 
     def __loadQuery(self):
-        """returns a Query object with loaded ruleset"""
+        """return a Query object with loaded ruleset"""
         return Query(
             "select ruleset, list, position, name, definition, points, doubles, limits, parameter from rule "
             "where ruleset=? order by list,position", (self.rulesetId,))
 
     def toList(self):
-        """returns entire ruleset encoded in a string"""
+        """return entire ruleset encoded in a string"""
         self.load()
         result = [[self.rulesetId, self.hash, self.name, self.description]]
         result.extend(self.ruleRecord(x) for x in self.allRules)
@@ -507,22 +486,22 @@ into a situation where you have to pay a penalty"""))
 
     def findUniqueOption(self, action):
         """return first rule with option"""
-        rulesWithAction = list(x for x in self.allRules if action in x.options)
+        rulesWithAction = [x for x in self.allRules if action in x.options]
         assert len(rulesWithAction) < 2, '%s has too many matching rules for %s' % (
             str(self), action)
         if rulesWithAction:
             return rulesWithAction[0]
+        return None
 
     def filterRules(self, attrName):
-        """returns all my Rule classes having attribute attrName"""
+        """return all my Rule classes having attribute attrName"""
         if attrName not in self.__filteredLists:
-            self.__filteredLists[attrName] = list(
-                x for x in self.allRules if hasattr(x, attrName))
+            self.__filteredLists[attrName] = [x for x in self.allRules if hasattr(x, attrName)]
         return self.__filteredLists[attrName]
 
     @staticmethod
     def newId(minus=False):
-        """returns an unused ruleset id. This is not multi user safe."""
+        """return an unused ruleset id. This is not multi user safe."""
         func = 'min(id)-1' if minus else 'max(id)+1'
         result = -1 if minus else 1
         records = Query("select %s from ruleset" % func).records
@@ -557,7 +536,7 @@ into a situation where you have to pay a penalty"""))
         return newId, newName
 
     def clone(self):
-        """returns a clone of self, unloaded"""
+        """return a clone of self, unloaded"""
         return Ruleset(self.rulesetId)
 
     def __str__(self):
@@ -581,6 +560,7 @@ into a situation where you have to pay a penalty"""))
             if rule in ruleList:
                 return ruleList
         assert False
+        return None
 
     def rename(self, newName):
         """renames the ruleset. returns True if done, False if not"""
@@ -609,7 +589,7 @@ into a situation where you have to pay a penalty"""))
         self.__hash = result.hexdigest()
 
     def ruleRecord(self, rule):
-        """returns the rule as tuple, prepared for use by sql. The first three
+        """return the rule as tuple, prepared for use by sql. The first three
         fields are the primary key."""
         score = rule.score
         ruleList = None
@@ -666,12 +646,12 @@ into a situation where you have to pay a penalty"""))
                 failSilent=True)
             cmd = 'INSERT INTO rule(ruleset, list, position, name, definition, ' \
                 'points, doubles, limits, parameter) VALUES(?,?,?,?,?,?,?,?,?)'
-            args = list(self.ruleRecord(x) for x in self.allRules)
+            args = [self.ruleRecord(x) for x in self.allRules]
             Query(cmd, args)
 
     @staticmethod
     def availableRulesets():
-        """returns all rulesets defined in the database plus all predefined rulesets"""
+        """return all rulesets defined in the database plus all predefined rulesets"""
         templateIds = (x[0]
                        for x in Query("SELECT id FROM ruleset WHERE id<0").records)
         result = [Ruleset(x) for x in templateIds]
@@ -682,7 +662,7 @@ into a situation where you have to pay a penalty"""))
 
     @staticmethod
     def selectableRulesets(server=None):
-        """returns all selectable rulesets for a new game.
+        """return all selectable rulesets for a new game.
         server is used to find the last ruleset used by us on that server, this
         ruleset will returned first in the list."""
         result = Ruleset.availableRulesets()
@@ -720,8 +700,8 @@ into a situation where you have to pay a penalty"""))
     def diff(self, other):
         """return a list of tuples. Every tuple holds one or two rules: tuple[0] is from self, tuple[1] is from other"""
         result = []
-        leftDict = dict((x.name, x) for x in self.allRules)
-        rightDict = dict((x.name, x) for x in other.allRules)
+        leftDict = {x.name: x for x in self.allRules}
+        rightDict = {x.name: x for x in other.allRules}
         left = set(leftDict.keys())
         right = set(rightDict.keys())
         for rule in left & right:
@@ -806,23 +786,21 @@ class Rule(RuleBase):
                         (id(hand) %
                          10000, cacheKey, result))
                 return result
-            else:
-                if Debug.ruleCache:
-                    if hand.ruleCache[cacheKey] != func(*args):
-                        hand.player.game.debug(
-                            'cacheKey=%s rule=%s func:%s args:%s' %
-                            (cacheKey, srcClass, func, args))
-                        hand.player.game.debug(
-                            '  hand:%s/%s' %
-                            (id(hand), hand))
-                        hand.player.game.debug(
-                            '  cached:%s ' %
-                            str(hand.ruleCache[cacheKey]))
-                        hand.player.game.debug(
-                            '    real:%s ' %
-                            str(func(*args)))
-                return hand.ruleCache[cacheKey]
-            return result
+            if Debug.ruleCache:
+                if hand.ruleCache[cacheKey] != func(*args):
+                    hand.player.game.debug(
+                        'cacheKey=%s rule=%s func:%s args:%s' %
+                        (cacheKey, srcClass, func, args))
+                    hand.player.game.debug(
+                        '  hand:%s/%s' %
+                        (id(hand), hand))
+                    hand.player.game.debug(
+                        '  cached:%s ' %
+                        str(hand.ruleCache[cacheKey]))
+                    hand.player.game.debug(
+                        '    real:%s ' %
+                        str(func(*args)))
+            return hand.ruleCache[cacheKey]
         return classmethod(wrapper) if clsMethod else staticmethod(wrapper)
 
     def __init__(self, name, definition='', points=0, doubles=0, limits=0,
@@ -940,7 +918,7 @@ class Rule(RuleBase):
         return '%s: %s %s' % (self.name, self.definition, self.score)
 
     def i18nStr(self):
-        """returns a human readable string with the content"""
+        """return a human readable string with the content"""
         return self.score.i18nStr()
 
     @staticmethod
@@ -986,7 +964,7 @@ class ParameterRule(RuleBase):
         return result
 
     def i18nStr(self):
-        """returns a human readable string with the content"""
+        """return a human readable string with the content"""
         return str(self.parameter)
 
 
@@ -1013,6 +991,7 @@ class IntRule(ParameterRule):
             return i18nc(
                 'wrong value for rule', '%1: %2 is too small, minimal value is %3',
                 i18n(self.name), self.parName, self.minimum)
+        return None
 
 
 class BoolRule(ParameterRule):
@@ -1043,7 +1022,7 @@ class PredefinedRuleset(Ruleset):
     """special code for loading rules from program code instead of from the database"""
 
     classes = set()  # only those will be playable
-    preRulesets = None
+    preRulesets = []
 
     def __init__(self, name=None):
         Ruleset.__init__(self, name or 'general predefined ruleset')
@@ -1051,14 +1030,13 @@ class PredefinedRuleset(Ruleset):
     @staticmethod
     def rulesets():
         """a list of instances for all predefined rulesets"""
-        if PredefinedRuleset.preRulesets is None:
-            PredefinedRuleset.preRulesets = list(x()
-                                                 for x in sorted(PredefinedRuleset.classes, key=lambda x: x.__name__))
+        if not PredefinedRuleset.preRulesets:
+            PredefinedRuleset.preRulesets = [
+                x() for x in sorted(PredefinedRuleset.classes, key=lambda x: x.__name__)]
         return PredefinedRuleset.preRulesets
 
     def rules(self):
         """here the predefined rulesets can define their rules"""
-        pass
 
     def clone(self):
         """return a clone, unloaded"""
