@@ -3,19 +3,8 @@
 """
 Copyright (C) 2009-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-Kajongg is free software you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation either version 2 of the License, or
-(at your option) any later version.
+SPDX-License-Identifier: GPL-2.0
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 import datetime
@@ -45,7 +34,7 @@ class TablesModel(QAbstractTableModel):
     """a model for our tables"""
 
     def __init__(self, tables, parent=None):
-        super(TablesModel, self).__init__(parent)
+        super().__init__(parent)
         self.tables = tables
         assert isinstance(tables, list)
 
@@ -57,10 +46,9 @@ class TablesModel(QAbstractTableModel):
             if orientation == Qt.Horizontal:
                 if section in [3, 4]:
                     return int(Qt.AlignLeft)
-                else:
-                    return int(Qt.AlignHCenter | Qt.AlignVCenter)
+                return int(Qt.AlignHCenter | Qt.AlignVCenter)
         if role != Qt.DisplayRole:
-            return
+            return None
         if orientation != Qt.Horizontal:
             return int(section + 1)
         result = ''
@@ -80,7 +68,7 @@ class TablesModel(QAbstractTableModel):
             return 0
         return len(self.tables)
 
-    def columnCount(self, dummyParent=None):  # pylint: disable=no-self-use
+    def columnCount(self, unusedParent=None):  # pylint: disable=no-self-use
         """for now we only have id (invisible), id (visible), players,
         status, ruleset.name.
         id(invisible) always holds the real id, also 1000 for suspended tables.
@@ -89,7 +77,7 @@ class TablesModel(QAbstractTableModel):
 
     def data(self, index, role=Qt.DisplayRole):
         """score table"""
-        # pylint: disable=too-many-branches,too-many-locals,redefined-variable-type
+        # pylint: disable=too-many-branches,too-many-locals
         result = None
         if role == Qt.TextAlignmentRole:
             if index.column() == 0:
@@ -143,7 +131,7 @@ class TablesModel(QAbstractTableModel):
                     result = i18n((table.myRuleset if table.myRuleset else table.ruleset).name)
                 elif role == Qt.ForegroundRole:
                     palette = KApplication.palette()
-                    color = palette.windowText() if table.myRuleset else 'red'
+                    color = palette.windowText().color() if table.myRuleset else 'red'
                     result = QColor(color)
         return result
 
@@ -176,7 +164,7 @@ class TableList(QWidget):
     # pylint: disable=too-many-instance-attributes
 
     def __init__(self, client):
-        super(TableList, self).__init__(None)
+        super().__init__(None)
         self.autoStarted = False
         self.client = client
         self.setObjectName('TableList')
@@ -243,7 +231,7 @@ class TableList(QWidget):
         StateSaver(self, self.view.horizontalHeader())
         self.updateButtonsForTable(None)
 
-    def hideEvent(self, dummyEvent):  # pylint: disable=no-self-use
+    def hideEvent(self, unusedEvent):  # pylint: disable=no-self-use
         """table window hides"""
         scene = Internal.scene
         if scene:
@@ -357,17 +345,18 @@ class TableList(QWidget):
                 "For chatting with others on this table, "
                 "please first take a seat"))
 
-    def selectionChanged(self, selected, dummyDeselected):
+    def selectionChanged(self, selected, unusedDeselected):
         """update button states according to selection"""
         if selected.indexes():
             self.selectTable(selected.indexes()[0].row())
 
     def selectedTable(self):
-        """returns the selected table"""
+        """return the selected table"""
         if self.view.selectionModel():
             index = self.view.selectionModel().currentIndex()
             if index.isValid() and self.view.model():
                 return self.view.model().tables[index.row()]
+        return None
 
     def compareRuleset(self):
         """compare the ruleset of this table against ours"""
@@ -396,12 +385,11 @@ class TableList(QWidget):
         """copy chatWindows from the old table list which will be
         thrown away"""
         if self.view.model():
-            chatWindows = dict((x.tableid, x.chatWindow)
-                               for x in self.view.model().tables)
-            unusedWindows = set(x.chatWindow for x in self.view.model().tables)
+            chatWindows = {x.tableid: x.chatWindow for x in self.view.model().tables}
+            unusedWindows = {x.chatWindow for x in self.view.model().tables}
             for table in tables:
                 table.chatWindow = chatWindows.get(table.tableid, None)
-                unusedWindows -= set([table.chatWindow])
+                unusedWindows -= {table.chatWindow}
             for unusedWindow in unusedWindows:
                 if unusedWindow:
                     unusedWindow.hide()
@@ -421,8 +409,8 @@ class TableList(QWidget):
             self.requestedNewTable = False
             model = self.view.model()
             if model:
-                oldIds = set(x.tableid for x in model.tables)
-                newIds = sorted(list(set(x.tableid for x in tables) - oldIds))
+                oldIds = {x.tableid for x in model.tables}
+                newIds = sorted({x.tableid for x in tables} - oldIds)
                 if newIds:
                     return newIds[0]
         if self.selectedTable():
