@@ -3,19 +3,8 @@
 """
  Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-Kajongg is free software you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation either version 2 of the License, or
-(at your option) any later version.
+SPDX-License-Identifier: GPL-2.0
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 from qt import Qt, QRectF, QPointF, QSizeF, QSize
@@ -44,7 +33,7 @@ class UITile(AnimatedMixin, QGraphicsObject, StrMixin):
     clsUid = 0
 
     def __init__(self, tile, xoffset=0.0, yoffset=0.0, level=0):
-        super(UITile, self).__init__()
+        super().__init__()
         if not isinstance(tile, Tile):
             tile = Tile(tile)
         UITile.clsUid += 1
@@ -165,23 +154,27 @@ class UITile(AnimatedMixin, QGraphicsObject, StrMixin):
                 else self.tileset.faceSize)
         return self._boundingRect
 
-    def facePos(self):
-        """returns the face position relative to the tile
+    def facePos(self, showShadows=None):
+        """return the face position relative to the tile
         depend on tileset, lightSource and shadow"""
-        return self.board.tileFacePos()
+        if showShadows is None:
+            showShadows = Internal.Preferences.showShadows
+        return self.board.tileFacePos(showShadows)
 
     def showFace(self):
         """should we show face for this tile?"""
         return self.tile.isKnown
 
-    def __elementId(self):
-        """returns the SVG element id of the tile"""
-        if not Internal.Preferences.showShadows:
+    def __elementId(self, showShadows=None):
+        """return the SVG element id of the tile"""
+        if showShadows is None:
+            showShadows = Internal.Preferences.showShadows
+        if not showShadows:
             return "TILE_2"
         lightSourceIndex = LIGHTSOURCES.index(self.board.rotatedLightSource())
         return "TILE_{}".format(lightSourceIndex % 4 + 1)
 
-    def paint(self, painter, dummyOption, dummyWidget=None):
+    def paint(self, painter, unusedOption, unusedWidget=None):
         """paint the entire tile.
         I tried to cache a pixmap for the tile and darkener but without face,
         but that actually made it slower."""
@@ -216,17 +209,15 @@ class UITile(AnimatedMixin, QGraphicsObject, StrMixin):
             painter.drawLine(QPointF(0.0, 0.0), QPointF(width, height))
             painter.drawLine(QPointF(width, 0.0), QPointF(0.0, height))
 
-    def pixmapFromSvg(self, pmapSize=None, withBorders=None):
-        """returns a pixmap with default size as given in SVG
+    def pixmapFromSvg(self, pmapSize, withBorders=None):
+        """return a pixmap with default size as given in SVG
         and optional borders/shadows"""
         if withBorders is None:
             withBorders = Internal.Preferences.showShadows
         if withBorders:
-            wantSize = self.tileset.tileSize.toSize()
+            originalSize = self.tileset.tileSize.toSize()
         else:
-            wantSize = self.tileset.faceSize.toSize()
-        if not pmapSize:
-            pmapSize = wantSize
+            originalSize = self.tileset.faceSize.toSize()
         result = QPixmap(pmapSize)
         result.fill(Qt.transparent)
         painter = QPainter(result)
@@ -235,24 +226,25 @@ class UITile(AnimatedMixin, QGraphicsObject, StrMixin):
                 'painter is not active. Wanted size: %s' %
                 str(pmapSize))
         try:
-            xScale = float(pmapSize.width()) / wantSize.width()
-            yScale = float(pmapSize.height()) / wantSize.height()
+            xScale = float(pmapSize.width()) / originalSize.width()
+            yScale = float(pmapSize.height()) / originalSize.height()
         except ZeroDivisionError:
             xScale = 1
             yScale = 1
+        # draw the tile too far to the left/upper side such that its shadow is outside of the print region
         if not withBorders:
             painter.scale(*self.tileset.tileFaceRelation())
-            painter.translate(-self.facePos())
         renderer = self.tileset.renderer()
-        renderer.render(painter, self.__elementId())
+        renderer.render(painter, self.__elementId(showShadows=withBorders))
         painter.resetTransform()
         self._drawDarkness(painter)
         if self.showFace():
             faceSize = self.tileset.faceSize.toSize()
             faceSize = QSize(
-                faceSize.width() * xScale,
-                faceSize.height() * yScale)
-            painter.translate(self.facePos())
+                int(faceSize.width() * xScale),
+                int(faceSize.height() * yScale))
+            painter.resetTransform()
+            painter.translate(self.facePos(withBorders))
             renderer.render(painter, self.tileset.svgName[self.tile.exposed],
                             QRectF(QPointF(), QSizeF(faceSize)))
         return result
@@ -272,12 +264,11 @@ class UITile(AnimatedMixin, QGraphicsObject, StrMixin):
         sorter = dirs[dirs.index(sortDir) + sceneRotation(self.__board) // 90]
         if sorter == Qt.Key_Down:
             return self.xoffset * 100 + self.yoffset
-        elif sorter == Qt.Key_Up:
-            return -self.xoffset * 100 - self.yoffset
-        elif sorter == Qt.Key_Left:
-            return -self.yoffset * 100 - self.xoffset
-        else:
-            return self.yoffset * 100 + self.xoffset
+        if sorter == Qt.Key_Up:
+            return -(self.xoffset * 100 + self.yoffset)
+        if sorter == Qt.Key_Left:
+            return -(self.yoffset * 100 + self.xoffset)
+        return self.yoffset * 100 + self.xoffset
 
     def setBoard(self, board, xoffset=None, yoffset=None, level=0):
         """change Position of tile in board"""
@@ -313,6 +304,7 @@ class UITile(AnimatedMixin, QGraphicsObject, StrMixin):
     def tile(self, value):  # pylint: disable=arguments-differ
         """set tile name and update display"""
         if value is not self._tile:
+            assert not self._tile.isKnown or (self._tile.exposed == value.exposed)
             self._tile = value
             self.setDrawingOrder() # because known tiles are above unknown tiles
             self.update()
@@ -426,9 +418,9 @@ class UIMeld(list):
             self.extend(newContent)
         elif isinstance(newContent, UITile):
             self.append(newContent)
-        assert len(self), newContent
+        assert self, newContent
 
     @property
     def meld(self):
-        """returns a logical meld"""
+        """return a logical meld"""
         return Meld(x.tile for x in self)
