@@ -3,19 +3,8 @@
 """
 Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-Kajongg is free software you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation either version 2 of the License, or
-(at your option) any later version.
+SPDX-License-Identifier: GPL-2.0
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 import weakref
@@ -27,7 +16,6 @@ from tile import Tile, elements
 class WallEmpty(Exception):
 
     """exception when trying to get a tile off the empty wall"""
-    pass
 
 
 class KongBox:
@@ -91,10 +79,9 @@ class Wall(StrMixin):
         assert isinstance(element, Tile), element
         if isinstance(tile, Tile):
             return element
-        else:
-            # tile is UITile
-            tile.tile = element
-            return tile
+        # tile is UITile
+        tile.tile = element
+        return tile
 
     def deal(self, tiles=None, deadEnd=False):
         """deal tiles. May raise WallEmpty.
@@ -111,15 +98,13 @@ class Wall(StrMixin):
                 raise WallEmpty
             dealTiles = self.living[:count]
             self.living = self.living[count:]
-        return list(self.__nameTile(*x)
-                    for x in zip(dealTiles, tiles))
+        return [self.__nameTile(*x) for x in zip(dealTiles, tiles)]
 
     def build(self, shuffleFirst=False):
         """virtual: build visible wall"""
 
     def _placeLooseTiles(self, deferredResult=None):
         """to be done only for UIWall"""
-        pass
 
     def decorate4(self, deferredResult=None): # pylint: disable=unused-argument
         """virtual: show player info on the wall"""
